@@ -1,7 +1,7 @@
 // drumkv1.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -29,9 +29,11 @@
 #include "drumkv1_list.h"
 
 #include "drumkv1_fx.h"
-
 #include "drumkv1_reverb.h"
 
+#include "drumkv1_config.h"
+#include "drumkv1_programs.h"
+
 
 #ifdef CONFIG_DEBUG_0
 #include <stdio.h>
@@ -758,7 +760,7 @@ class drumkv1_impl
 {
 public:
 
-	drumkv1_impl(uint16_t iChannels, uint32_t iSampleRate);
+	drumkv1_impl(drumkv1 *pDrumk, uint16_t iChannels, uint32_t iSampleRate);
 
 	~drumkv1_impl();
 
@@ -786,7 +788,12 @@ public:
 	bool isReverse() const;
 
 	void setParamPort(drumkv1::ParamIndex index, float *pfParam = 0);
-	float *paramPort(drumkv1::ParamIndex index);
+	float *paramPort(drumkv1::ParamIndex index) const;
+
+	void setParamValue(drumkv1::ParamIndex index, float fValue);
+	float paramValue(drumkv1::ParamIndex index) const;
+
+	drumkv1_programs *programs();
 
 	void process_midi(uint8_t *data, uint32_t size);
 	void process(float **ins, float **outs, uint32_t nframes);
@@ -826,6 +833,9 @@ protected:
 
 private:
 
+	drumkv1_config   m_config;
+	drumkv1_programs m_programs;
+
 	uint16_t m_iChannels;
 	uint32_t m_iSampleRate;
 
@@ -866,7 +876,9 @@ private:
 
 // synth engine constructor
 
-drumkv1_impl::drumkv1_impl ( uint16_t iChannels, uint32_t iSampleRate )
+drumkv1_impl::drumkv1_impl (
+	drumkv1 *pDrumk, uint16_t iChannels, uint32_t iSampleRate )
+	: m_programs(pDrumk)
 {
 	// allocate voice pool.
 	m_voices = new drumkv1_voice * [MAX_VOICES];
@@ -898,6 +910,9 @@ drumkv1_impl::drumkv1_impl ( uint16_t iChannels, uint32_t iSampleRate )
 	m_del.bpmsync0 = 0.0f;
 	m_del.bpm0 = 0;
 
+	// load programs database...
+	m_config.loadPrograms(&m_programs);
+
 	// number of channels
 	setChannels(iChannels);
 
@@ -925,6 +940,12 @@ drumkv1_impl::drumkv1_impl ( uint16_t iChannels, uint32_t iSampleRate )
 
 drumkv1_impl::~drumkv1_impl (void)
 {
+#if 0
+	// DO NOT save programs database here:
+	// prevent multi-instance clash...
+	m_config.savePrograms(&m_programs);
+#endif
+
 	// deallocate sample filenames
 	setSampleFile(0);
 
@@ -1190,7 +1211,7 @@ void drumkv1_impl::setParamPort ( drumkv1::ParamIndex index, float *pfParam )
 }
 
 
-float *drumkv1_impl::paramPort ( drumkv1::ParamIndex index )
+float *drumkv1_impl::paramPort ( drumkv1::ParamIndex index ) const
 {
 	float *pfParam = 0;
 
@@ -1240,6 +1261,21 @@ float *drumkv1_impl::paramPort ( drumkv1::ParamIndex index )
 }
 
 
+void drumkv1_impl::setParamValue ( drumkv1::ParamIndex index, float fValue )
+{
+	float *pfParamPort = paramPort(index);
+	if (pfParamPort)
+		*pfParamPort = fValue;
+}
+
+
+float drumkv1_impl::paramValue ( drumkv1::ParamIndex index ) const
+{
+	float *pfParamPort = paramPort(index);
+	return (pfParamPort ? *pfParamPort : 0.0f);
+}
+
+
 // handle midi input
 
 void drumkv1_impl::process_midi ( uint8_t *data, uint32_t size )
@@ -1261,6 +1297,10 @@ void drumkv1_impl::process_midi ( uint8_t *data, uint32_t size )
 	const int status = (data[0] & 0xf0);
 	const int key    = (data[1] & 0x7f);
 
+	// program change
+	if (status == 0xc0)
+		m_programs.prog_change(key);
+	else
 	if (status == 0xd0) {
 		// channel aftertouch
 		m_ctl.pressure = float(key) / 127.0f;
@@ -1275,7 +1315,7 @@ void drumkv1_impl::process_midi ( uint8_t *data, uint32_t size )
 	// note on
 	if (status == 0x90 && value > 0) {
 		drumkv1_voice *pv = m_notes[key];
-		if (pv && int(*m_def.noteoff) > 0) {
+		if (pv && !(int(*m_def.noteoff) > 0)) {
 			drumkv1_elem *elem = pv->elem;
 			// retrigger fast release
 			elem->dcf1.env.note_off_fast(&pv->dcf1_env);
@@ -1357,7 +1397,11 @@ void drumkv1_impl::process_midi ( uint8_t *data, uint32_t size )
 	}
 	// control change
 	else if (status == 0xb0) {
-		switch (key) {
+	switch (key) {
+		case 0x00:
+			// bank-select MSB (cc#0)
+			m_programs.bank_select_msb(value);
+			break;
 		case 0x01:
 			// modulation wheel (cc#1)
 			m_ctl.modwheel = *m_def.modwheel * float(value) / 127.0f;
@@ -1370,6 +1414,10 @@ void drumkv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			// channel panning (cc#10)
 			m_ctl.panning = float(value - 64) / 64.0f;
 			break;
+		case 0x20:
+			// bank-select LSB (cc#32)
+			m_programs.bank_select_lsb(value);
+			break;
 		case 0x78:
 			// all sound off (cc#120)
 			allSoundOff();
@@ -1514,6 +1562,14 @@ void drumkv1_impl::reset (void)
 }
 
 
+// programs accessor
+
+drumkv1_programs *drumkv1_impl::programs (void)
+{
+	return &m_programs;
+}
+
+
 // synthesize
 
 void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
@@ -1740,13 +1796,14 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 }
 
 
+
 //-------------------------------------------------------------------------
 // drumkv1 - decl.
 //
 
 drumkv1::drumkv1 ( uint16_t iChannels, uint32_t iSampleRate )
 {
-	m_pImpl = new drumkv1_impl(iChannels, iSampleRate);
+	m_pImpl = new drumkv1_impl(this, iChannels, iSampleRate);
 }
 
 
@@ -1852,6 +1909,17 @@ float *drumkv1::paramPort ( ParamIndex index ) const
 }
 
 
+void drumkv1::setParamValue ( ParamIndex index, float fValue )
+{
+	m_pImpl->setParamValue(index, fValue);
+}
+
+float drumkv1::paramValue ( ParamIndex index ) const
+{
+	return m_pImpl->paramValue(index);
+}
+
+
 void drumkv1::process_midi ( uint8_t *data, uint32_t size )
 {
 #ifdef CONFIG_DEBUG_0
@@ -1879,6 +1947,14 @@ void drumkv1::resetParamValues ( bool bSwap )
 }
 
 
+// programs accessor
+
+drumkv1_programs *drumkv1::programs (void) const
+{
+	return m_pImpl->programs();
+}
+
+
 // all reset clear
 
 void drumkv1::reset (void)
