@@ -1,7 +1,7 @@
 // drumkv1_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -19,9 +19,8 @@
 
 *****************************************************************************/
 
-#include "drumkv1_config.h"
-
 #include "drumkv1_jack.h"
+#include "drumkv1_config.h"
 #include "drumkv1_param.h"
 
 #include <jack/midiport.h>
@@ -148,7 +147,7 @@ drumkv1_jack::drumkv1_jack (void) : drumkv1(2)
 
 	m_ins = m_outs = NULL;
 
-	::memset(m_params, 0, NUM_PARAMS * sizeof(float));
+	::memset(m_params, 0, drumkv1::NUM_PARAMS * sizeof(float));
 
 #ifdef CONFIG_JACK_MIDI
 	m_midi_in = NULL;
@@ -187,7 +186,7 @@ int drumkv1_jack::process ( jack_nframes_t nframes )
 	if (!m_activated)
 		return 0;
 
-	const uint16_t nchannels = channels();
+	const uint16_t nchannels = drumkv1::channels();
 	float *ins[nchannels], *outs[nchannels];
 	for (uint16_t k = 0; k < nchannels; ++k) {
 		ins[k]  = static_cast<float *> (
@@ -196,16 +195,16 @@ int drumkv1_jack::process ( jack_nframes_t nframes )
 			::jack_port_get_buffer(m_audio_outs[k], nframes));
 	}
 
-	const float *bpmsync = paramPort(drumkv1::DEL1_BPMSYNC);
-	if (bpmsync && *bpmsync > 0.0f) {
-		float *bpmhost = paramPort(drumkv1::DEL1_BPMHOST);
-		if (bpmhost) {
+	const float bpm_sync = paramValue(drumkv1::DEL1_BPMSYNC);
+	if (bpm_sync > 0.0f) {
+		const float bpm_host = paramValue(drumkv1::DEL1_BPMHOST);
+		if (bpm_host > 0.0f) {
 			jack_position_t pos;
 			jack_transport_query(m_client, &pos);
 			if (pos.valid & JackPositionBBT) {
 				const float bpm = float(pos.beats_per_minute);
-				if (::fabs(*bpmhost - bpm) > 0.01f)
-					*bpmhost = bpm;
+				if (::fabs(bpm_host - bpm) > 0.01f)
+					setParamValue(drumkv1::DEL1_BPMHOST, bpm);
 			}
 		}
 	}
@@ -288,7 +287,7 @@ void drumkv1_jack::open ( const char *client_id )
 //	drumkv1::reset();
 
 	// register audio ports & buffers
-	uint16_t nchannels = channels();
+	uint16_t nchannels = drumkv1::channels();
 
 	m_audio_ins  = new jack_port_t * [nchannels];
 	m_audio_outs = new jack_port_t * [nchannels];
@@ -410,7 +409,7 @@ void drumkv1_jack::close (void)
 #endif
 
 	// unregister audio ports
-	const uint16_t nchannels = channels();
+	const uint16_t nchannels = drumkv1::channels();
 
 	for (uint16_t k = 0; k < nchannels; ++k) {
 		if (m_audio_outs && m_audio_outs[k]) {
@@ -592,7 +591,7 @@ void drumkv1_jack::sessionEvent ( void *pvSessionArg )
 
 
 //-------------------------------------------------------------------------
-// drumkv1_application -- Singleton application instance.
+// drumkv1_jack_application -- Singleton application instance.
 //
 
 #include "drumkv1widget_jack.h"
@@ -606,7 +605,7 @@ void drumkv1_jack::sessionEvent ( void *pvSessionArg )
 
 
 // Constructor.
-drumkv1_application::drumkv1_application ( int& argc, char **argv )
+drumkv1_jack_application::drumkv1_jack_application ( int& argc, char **argv )
 	: QObject(NULL), m_pApp(NULL), m_bGui(true),
 		m_pDrumk(NULL), m_pWidget(NULL)
 	  #ifdef CONFIG_NSM
@@ -633,7 +632,7 @@ drumkv1_application::drumkv1_application ( int& argc, char **argv )
 
 
 // Destructor.
-drumkv1_application::~drumkv1_application (void)
+drumkv1_jack_application::~drumkv1_jack_application (void)
 {
 #ifdef CONFIG_NSM
 	if (m_pNsmClient) delete m_pNsmClient;
@@ -645,7 +644,7 @@ drumkv1_application::~drumkv1_application (void)
 
 
 // Argument parser method.
-bool drumkv1_application::parse_args (void)
+bool drumkv1_jack_application::parse_args (void)
 {
 	QTextStream out(stderr);
 
@@ -677,7 +676,7 @@ bool drumkv1_application::parse_args (void)
 
 
 // Startup methods.
-bool drumkv1_application::setup (void)
+bool drumkv1_jack_application::setup (void)
 {
 	if (m_pApp == NULL)
 		return false;
@@ -733,7 +732,7 @@ bool drumkv1_application::setup (void)
 
 
 // Facade method.
-int drumkv1_application::exec (void)
+int drumkv1_jack_application::exec (void)
 {
 	return (setup() ? m_pApp->exec() : 1);
 }
@@ -741,7 +740,7 @@ int drumkv1_application::exec (void)
 
 #ifdef CONFIG_NSM
 
-void drumkv1_application::openSession (void)
+void drumkv1_jack_application::openSession (void)
 {
 	if (m_pDrumk == NULL)
 		return;
@@ -787,7 +786,7 @@ void drumkv1_application::openSession (void)
 		m_pNsmClient->visible(m_pWidget->isVisible());
 }
 
-void drumkv1_application::saveSession (void)
+void drumkv1_jack_application::saveSession (void)
 {
 	if (m_pDrumk == NULL)
 		return;
@@ -814,7 +813,7 @@ void drumkv1_application::saveSession (void)
 }
 
 
-void drumkv1_application::showSession (void)
+void drumkv1_jack_application::showSession (void)
 {
 	if (m_pNsmClient == NULL)
 		return;
@@ -833,7 +832,7 @@ void drumkv1_application::showSession (void)
 	}
 }
 
-void drumkv1_application::hideSession (void)
+void drumkv1_jack_application::hideSession (void)
 {
 	if (m_pNsmClient == NULL)
 		return;
@@ -860,7 +859,7 @@ int main ( int argc, char *argv[] )
 {
 	Q_INIT_RESOURCE(drumkv1);
 
-	drumkv1_application app(argc, argv);
+	drumkv1_jack_application app(argc, argv);
 
 	return app.exec();
 }
