@@ -1,7 +1,7 @@
 // drumkv1_lv2.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -124,7 +124,7 @@ drumkv1_lv2::drumkv1_lv2 (
 		}
 	}
 
-	const uint16_t nchannels = channels();
+	const uint16_t nchannels = drumkv1::channels();
 	m_ins  = new float * [nchannels];
 	m_outs = new float * [nchannels];
 	for (uint16_t k = 0; k < nchannels; ++k)
@@ -158,7 +158,7 @@ void drumkv1_lv2::connect_port ( uint32_t port, void *data )
 		m_outs[1] = (float *) data;
 		break;
 	default:
-		setParamPort(ParamIndex(port - ParamBase), (float *) data);
+		drumkv1::setParamPort(drumkv1::ParamIndex(port - ParamBase), (float *) data);
 		break;
 	}
 }
@@ -166,7 +166,7 @@ void drumkv1_lv2::connect_port ( uint32_t port, void *data )
 
 void drumkv1_lv2::run ( uint32_t nframes )
 {
-	const uint16_t nchannels = channels();
+	const uint16_t nchannels = drumkv1::channels();
 	float *ins[nchannels], *outs[nchannels];
 	for (uint16_t k = 0; k < nchannels; ++k) {
 		ins[k]  = m_ins[k];
@@ -183,14 +183,14 @@ void drumkv1_lv2::run ( uint32_t nframes )
 				uint8_t *data = (uint8_t *) LV2_ATOM_BODY(&event->body);
 				const uint32_t nread = event->time.frames - ndelta;
 				if (nread > 0) {
-					process(ins, outs, nread);
+					drumkv1::process(ins, outs, nread);
 					for (uint16_t k = 0; k < nchannels; ++k) {
 						ins[k]  += nread;
 						outs[k] += nread;
 					}
 				}
 				ndelta = event->time.frames;
-				process_midi(data, event->body.size);
+				drumkv1::process_midi(data, event->body.size);
 			}
 			else
 			if (event->body.type == m_urids.atom_Blank ||
@@ -202,13 +202,13 @@ void drumkv1_lv2::run ( uint32_t nframes )
 					lv2_atom_object_get(object,
 						m_urids.time_beatsPerMinute, &atom, NULL);
 					if (atom && atom->type == m_urids.atom_Float) {
-						const float *bpmsync = paramPort(drumkv1::DEL1_BPMSYNC);
-						if (bpmsync && *bpmsync > 0.0f) {
-							float *bpmhost = paramPort(drumkv1::DEL1_BPMHOST);
-							if (bpmhost) {
+						const float bpm_sync = drumkv1::paramValue(drumkv1::DEL1_BPMSYNC);
+						if (bpm_sync > 0.0f) {
+							const float bpm_host = drumkv1::paramValue(drumkv1::DEL1_BPMHOST);
+							if (bpm_host > 0.0f) {
 								const float bpm	= ((LV2_Atom_Float *) atom)->body;
-								if (::fabs(*bpmhost - bpm) > 0.01f)
-									*bpmhost = bpm;
+								if (::fabs(bpm_host - bpm) > 0.01f)
+									drumkv1::setParamValue(drumkv1::DEL1_BPMHOST, bpm);
 							}
 						}
 					}
@@ -218,19 +218,19 @@ void drumkv1_lv2::run ( uint32_t nframes )
 	//	m_atom_sequence = NULL;
 	}
 
-	process(ins, outs, nframes - ndelta);
+	drumkv1::process(ins, outs, nframes - ndelta);
 }
 
 
 void drumkv1_lv2::activate (void)
 {
-	reset();
+	drumkv1::reset();
 }
 
 
 void drumkv1_lv2::deactivate (void)
 {
-	reset();
+	drumkv1::reset();
 }
 
 
@@ -252,11 +252,11 @@ static LV2_State_Status drumkv1_lv2_state_save ( LV2_Handle instance,
 	if (pPlugin == NULL)
 		return LV2_STATE_ERR_UNKNOWN;
 
-	uint32_t key = pPlugin->urid_map(DRUMKV1_LV2_PREFIX "state");
+	const uint32_t key = pPlugin->urid_map(DRUMKV1_LV2_PREFIX "state");
 	if (key == 0)
 		return LV2_STATE_ERR_NO_PROPERTY;
 
-	uint32_t type = pPlugin->urid_map(LV2_ATOM__Chunk);
+	const uint32_t type = pPlugin->urid_map(LV2_ATOM__Chunk);
 	if (type == 0)
 		return LV2_STATE_ERR_BAD_TYPE;
 #if 0
@@ -288,11 +288,11 @@ static LV2_State_Status drumkv1_lv2_state_restore ( LV2_Handle instance,
 	if (pPlugin == NULL)
 		return LV2_STATE_ERR_UNKNOWN;
 
-	uint32_t key = pPlugin->urid_map(DRUMKV1_LV2_PREFIX "state");
+	const uint32_t key = pPlugin->urid_map(DRUMKV1_LV2_PREFIX "state");
 	if (key == 0)
 		return LV2_STATE_ERR_NO_PROPERTY;
 
-	uint32_t chunk_type = pPlugin->urid_map(LV2_ATOM__Chunk);
+	const uint32_t chunk_type = pPlugin->urid_map(LV2_ATOM__Chunk);
 	if (chunk_type == 0)
 		return LV2_STATE_ERR_BAD_TYPE;
 
@@ -324,7 +324,7 @@ static LV2_State_Status drumkv1_lv2_state_restore ( LV2_Handle instance,
 			drumkv1_param::loadElements(pPlugin, eElements, mapPath);
 	}
 
-	drumkv1_sched::sync_notify();
+	drumkv1_sched::sync_notify(drumkv1_sched::Sample);
 
 	return LV2_STATE_SUCCESS;
 }
@@ -337,6 +337,44 @@ static const LV2_State_Interface drumkv1_lv2_state_interface =
 };
 
 
+#ifdef CONFIG_LV2_PROGRAMS
+
+#include "drumkv1_programs.h"
+
+const LV2_Program_Descriptor *drumkv1_lv2::get_program ( uint32_t index )
+{
+	drumkv1_programs *pPrograms = drumkv1::programs();
+	const drumkv1_programs::Banks& banks = pPrograms->banks();
+	drumkv1_programs::Banks::ConstIterator bank_iter = banks.constBegin();
+	const drumkv1_programs::Banks::ConstIterator& bank_end = banks.constEnd();
+	for (uint32_t i = 0; bank_iter != bank_end; ++bank_iter) {
+		drumkv1_programs::Bank *pBank = bank_iter.value();
+		const drumkv1_programs::Progs& progs = pBank->progs();
+		drumkv1_programs::Progs::ConstIterator prog_iter = progs.constBegin();
+		const drumkv1_programs::Progs::ConstIterator& prog_end = progs.constEnd();
+		for ( ; prog_iter != prog_end; ++prog_iter, ++i) {
+			drumkv1_programs::Prog *pProg = prog_iter.value();
+			if (i >= index) {
+				m_aProgramName = pProg->name().toUtf8();
+				m_program.bank = pBank->id();
+				m_program.program = pProg->id();
+				m_program.name = m_aProgramName.constData();
+				return &m_program;
+			}
+		}
+	}
+
+	return NULL;
+}
+
+void drumkv1_lv2::select_program ( uint32_t bank, uint32_t program )
+{
+	drumkv1::programs()->select_program(bank, program);
+}
+
+#endif	// CONFIG_LV2_PROGRAMS
+
+
 //-------------------------------------------------------------------------
 // drumkv1_lv2 - LV2 desc.
 //
@@ -390,8 +428,41 @@ static void drumkv1_lv2_cleanup ( LV2_Handle instance )
 }
 
 
+#ifdef CONFIG_LV2_PROGRAMS
+
+static const LV2_Program_Descriptor *drumkv1_lv2_programs_get_program (
+	LV2_Handle instance, uint32_t index )
+{
+	drumkv1_lv2 *pPlugin = static_cast<drumkv1_lv2 *> (instance);
+	if (pPlugin)
+		return pPlugin->get_program(index);
+	else
+		return NULL;
+}
+
+static void drumkv1_lv2_programs_select_program (
+	LV2_Handle instance, uint32_t bank, uint32_t program )
+{
+	drumkv1_lv2 *pPlugin = static_cast<drumkv1_lv2 *> (instance);
+	if (pPlugin)
+		pPlugin->select_program(bank, program);
+}
+
+static const LV2_Programs_Interface drumkv1_lv2_programs_interface =
+{
+	drumkv1_lv2_programs_get_program,
+	drumkv1_lv2_programs_select_program,
+};
+
+#endif	// CONFIG_LV2_PROGRAMS
+
 static const void *drumkv1_lv2_extension_data ( const char *uri )
 {
+#ifdef CONFIG_LV2_PROGRAMS
+	if (::strcmp(uri, LV2_PROGRAMS__Interface) == 0)
+		return (void *) &drumkv1_lv2_programs_interface;
+	else
+#endif
 	if (::strcmp(uri, LV2_STATE__interface))
 		return NULL;
 
