@@ -1,7 +1,7 @@
 // drumkv1_param.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -19,8 +19,8 @@
 
 *****************************************************************************/
 
-#include "drumkv1_config.h"
 #include "drumkv1_param.h"
+#include "drumkv1_config.h"
 
 #include <QHash>
 
@@ -148,6 +148,8 @@ void drumkv1_param::loadElements (
 	if (pDrumk == NULL)
 		return;
 
+	pDrumk->clearElements();
+
 	static QHash<QString, drumkv1::ParamIndex> s_hash;
 	if (s_hash.isEmpty()) {
 		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i)
@@ -172,6 +174,7 @@ void drumkv1_param::loadElements (
 				if (eChild.tagName() == "sample") {
 				//	int index = eChild.attribute("index").toInt();
 					const QString& sFilename = eChild.text();
+					//	= QFileInfo(eChild.text()).canonicalFilePath();
 					element->setSampleFile(
 						mapPath.absolutePath(sFilename).toUtf8().constData());
 				}
@@ -189,7 +192,7 @@ void drumkv1_param::loadElements (
 							const QString& sName = eParam.attribute("name");
 							if (!sName.isEmpty() && s_hash.contains(sName))
 								index = s_hash.value(sName);
-							float fValue = eParam.text().toFloat();
+							const float fValue = eParam.text().toFloat();
 							element->setParamValue(index, fValue);
 						}
 					}
@@ -229,9 +232,8 @@ void drumkv1_param::saveElements (
 			eParam.setAttribute("index", QString::number(i));
 			eParam.setAttribute("name", drumkv1_default_params[i].name);
 			drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-			float *pfParam = element->paramPort(index);
-			eParam.appendChild(doc.createTextNode(QString::number(
-				pfParam ? *pfParam : element->paramValue(index))));
+			eParam.appendChild(doc.createTextNode(
+				QString::number(element->paramValue(index))));
 			eParams.appendChild(eParam);
 		}
 		eElement.appendChild(eParams);
@@ -246,7 +248,21 @@ void drumkv1_param::loadPreset ( drumkv1 *pDrumk, const QString& sFilename )
 	if (pDrumk == NULL)
 		return;
 
-	QFile file(sFilename);
+	QFileInfo fi(sFilename);
+	if (!fi.exists()) {
+		drumkv1_config *pConfig = drumkv1_config::getInstance();
+		if (pConfig) {
+			const QString& sPresetFile
+				= pConfig->presetFile(sFilename);
+			if (sPresetFile.isEmpty())
+				return;
+			fi.setFile(sPresetFile);
+			if (!fi.exists())
+				return;
+		}
+	}
+
+	QFile file(fi.filePath());
 	if (!file.open(QIODevice::ReadOnly))
 		return;
 
@@ -258,7 +274,6 @@ void drumkv1_param::loadPreset ( drumkv1 *pDrumk, const QString& sFilename )
 		}
 	}
 
-	const QFileInfo fi(sFilename);
 	const QDir currentDir(QDir::current());
 	QDir::setCurrent(fi.absolutePath());
 
@@ -293,14 +308,12 @@ void drumkv1_param::loadPreset ( drumkv1 *pDrumk, const QString& sFilename )
 									continue;
 								index = s_hash.value(sName);
 							}
-							float fParamValue = eParam.text().toFloat();
+							float fValue = eParam.text().toFloat();
 						#if 1//--legacy support < 0.3.0.4
-							if (index == drumkv1::DEL1_BPM && fParamValue < 3.6f)
-								fParamValue *= 100.0f;
+							if (index == drumkv1::DEL1_BPM && fValue < 3.6f)
+								fValue *= 100.0f;
 						#endif
-							float *pfParamPort = pDrumk->paramPort(index);
-							if (pfParamPort)
-								*pfParamPort = fParamValue;
+							pDrumk->setParamValue(index, fValue);
 						}
 					}
 				}
@@ -321,16 +334,13 @@ void drumkv1_param::savePreset ( drumkv1 *pDrumk, const QString& sFilename )
 	if (pDrumk == NULL)
 		return;
 
-	const QString& sPreset
-		= QFileInfo(sFilename).completeBaseName();
-
 	const QFileInfo fi(sFilename);
 	const QDir currentDir(QDir::current());
 	QDir::setCurrent(fi.absolutePath());
 
 	QDomDocument doc(DRUMKV1_TITLE);
 	QDomElement ePreset = doc.createElement("preset");
-	ePreset.setAttribute("name", sPreset);
+	ePreset.setAttribute("name", fi.completeBaseName());
 	ePreset.setAttribute("version", DRUMKV1_VERSION);
 
 	QDomElement eElements = doc.createElement("elements");
@@ -343,18 +353,14 @@ void drumkv1_param::savePreset ( drumkv1 *pDrumk, const QString& sFilename )
 		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 		eParam.setAttribute("index", QString::number(i));
 		eParam.setAttribute("name", drumkv1_param::paramName(index));
-		const float *pfParamPort = pDrumk->paramPort(index);
-		float fParamValue = 0.0f;
-		if (pfParamPort)
-			fParamValue = *pfParamPort;
-		eParam.appendChild(
-			doc.createTextNode(QString::number(fParamValue)));
+		const float fValue = pDrumk->paramValue(index);
+		eParam.appendChild(doc.createTextNode(QString::number(fValue)));
 		eParams.appendChild(eParam);
 	}
 	ePreset.appendChild(eParams);
 	doc.appendChild(ePreset);
 
-	QFile file(sFilename);
+	QFile file(fi.filePath());
 	if (file.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
 		QTextStream(&file) << doc.toString();
 		file.close();
