@@ -1,7 +1,7 @@
 // drumkv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -418,6 +418,9 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	QObject::connect(m_ui.Elements,
 		SIGNAL(itemDoubleClicked(int)),
 		SLOT(doubleClickElement()));
+	QObject::connect(m_ui.Elements,
+		SIGNAL(itemLoadSampleFile(const QString&, int)),
+		SLOT(loadSampleElement(const QString&)));
 
 	// Sample management...
 	QObject::connect(m_ui.Gen1Sample,
@@ -462,14 +465,10 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	QObject::connect(m_ui.TabBar, SIGNAL(currentChanged(int)),
 		m_ui.StackedWidget, SLOT(setCurrentIndex(int)));
 
-	drumkv1widget_config *pConfig = drumkv1widget_config::getInstance();
-	if (pConfig)
-		m_ui.helpUseNativeDialogsAction->setChecked(pConfig->bUseNativeDialogs);
-
 	// Menu actions
-	QObject::connect(m_ui.helpUseNativeDialogsAction,
+	QObject::connect(m_ui.helpConfigureAction,
 		SIGNAL(triggered(bool)),
-		SLOT(helpUseNativeDialogs(bool)));
+		SLOT(helpConfigure()));
 	QObject::connect(m_ui.helpAboutAction,
 		SIGNAL(triggered(bool)),
 		SLOT(helpAbout()));
@@ -479,8 +478,15 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	// Special sample update notifications (eg. reverse)
 	QObject::connect(m_sched_notifier,
-		SIGNAL(notify()),
-		SLOT(updateSampleNotify()));
+		SIGNAL(notify(int)),
+		SLOT(updateSchedNotify(int)));
+
+	// General knob/dial  behavior init...
+	drumkv1_config *pConfig = drumkv1_config::getInstance();
+	if (pConfig) {
+		drumkv1widget_dial::setDialMode(
+			drumkv1widget_dial::DialMode(pConfig->iKnobDialMode));
+	}
 
 	// Epilog.
 	// QWidget::adjustSize();
@@ -534,21 +540,18 @@ void drumkv1widget::setParamValue (
 
 float drumkv1widget::paramValue ( drumkv1::ParamIndex index ) const
 {
-	float fParamValue = 0.0f;
+	float fValue = 0.0f;
 
 	drumkv1widget_knob *pKnob = paramKnob(index);
 	if (pKnob) {
-		fParamValue = pKnob->value();
+		fValue = pKnob->value();
 	} else {
-		drumkv1 *pDrumk = instance();
-		if (pDrumk) {
-			const float *pParamPort = pDrumk->paramPort(index);
-			if (pParamPort)
-				fParamValue = *pParamPort;
-		}
+		drumkv1_ui *pDrumkUi = ui_instance();
+		if (pDrumkUi)
+			fValue = pDrumkUi->paramValue(index);
 	}
 
-	return fParamValue;
+	return fValue;
 }
 
 
@@ -561,6 +564,15 @@ void drumkv1widget::paramChanged ( float fValue )
 	drumkv1widget_knob *pKnob = qobject_cast<drumkv1widget_knob *> (sender());
 	if (pKnob) {
 		drumkv1::ParamIndex index = m_knobParams.value(pKnob);
+		// Save current element param value...
+		drumkv1_ui *pDrumkUi = ui_instance();
+		if (pDrumkUi) {
+			const int key = pDrumkUi->currentElement();
+			drumkv1_element *element = pDrumkUi->element(key);
+			if (element)
+				element->setParamValue(index, fValue);
+		}
+		// Proceed with regular formalities...
 		updateParam(index, fValue);
 		updateParamEx(index, fValue);
 		m_ui.StatusBar->showMessage(QString("%1 / %2: %3")
@@ -575,18 +587,17 @@ void drumkv1widget::paramChanged ( float fValue )
 // Update local tied widgets.
 void drumkv1widget::updateParamEx ( drumkv1::ParamIndex index, float fValue )
 {
-	drumkv1 *pDrumk = instance();
-	if (pDrumk == NULL)
-		return;
-
 	++m_iUpdate;
 
 	switch (index) {
-#if 0//--updateSampleNotify();
+#if 0//--updateSchedNotify(drumkv1_sched::Sample);
 	case drumkv1::GEN1_REVERSE: {
-		const bool bReverse = bool(fValue > 0.0f);
-		pDrumk->setReverse(bReverse);
-		updateSample(pDrumk->sample());
+		drumkv1_ui *pDrumkUi = ui_instance();
+		if (pDrumkUi) {
+			const bool bReverse = bool(fValue > 0.0f);
+			pDrumkUi->setReverse(bReverse);
+			updateSample(pDrumkUi->sample());
+		}
 		break;
 	}
 #endif
@@ -605,11 +616,11 @@ void drumkv1widget::updateParamEx ( drumkv1::ParamIndex index, float fValue )
 // Reset all param knobs to default values.
 void drumkv1widget::resetParams (void)
 {
-	drumkv1 *pDrumk = instance();
-	if (pDrumk == NULL)
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi == NULL)
 		return;
 
-	pDrumk->reset();
+	pDrumkUi->reset();
 
 	resetSwapParams();
 
@@ -640,10 +651,11 @@ void drumkv1widget::swapParams ( bool bOn )
 #endif
 //	resetParamKnobs(drumkv1::NUM_PARAMS);
 
-	drumkv1 *pDrumk = instance();
-	if (pDrumk) {
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi) {
 		// Save current element param values...
-		drumkv1_element *element = pDrumk->element(pDrumk->currentElement());
+		const int key = pDrumkUi->currentElement();
+		drumkv1_element *element = pDrumkUi->element(key);
 		if (element) {
 			for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
 				drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
@@ -655,7 +667,7 @@ void drumkv1widget::swapParams ( bool bOn )
 			}
 		}
 		// Swap all element params A/B...
-		pDrumk->resetParamValues(true);
+		pDrumkUi->resetParamValues(true);
 		// Retrieve current element param values...
 		if (element) {
 			for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
@@ -697,15 +709,13 @@ void drumkv1widget::updateParamValues ( uint32_t nparams )
 {
 	resetSwapParams();
 
-	drumkv1 *pDrumk = instance();
+	drumkv1_ui *pDrumkUi = ui_instance();
 
 	for (uint32_t i = 0; i < nparams; ++i) {
 		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-		float fValue = drumkv1_param::paramDefaultValue(index);
-		const float *pfParamPort
-			= (pDrumk ? pDrumk->paramPort(index) : NULL);
-		if (pfParamPort)
-			fValue = *pfParamPort;
+		const float fValue = (pDrumkUi
+			? pDrumkUi->paramValue(index)
+			: drumkv1_param::paramDefaultValue(index));
 		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
 	//	updateParamEx(index, fValue);
@@ -721,7 +731,7 @@ void drumkv1widget::resetParamValues ( uint32_t nparams )
 
 	for (uint32_t i = 0; i < nparams; ++i) {
 		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-		float fValue = drumkv1_param::paramDefaultValue(index);
+		const float fValue = drumkv1_param::paramDefaultValue(index);
 		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
 		m_params_ab[index] = fValue;
@@ -768,9 +778,9 @@ void drumkv1widget::newPreset (void)
 	resetParamKnobs(drumkv1::NUM_PARAMS);
 	resetParamValues(drumkv1::NUM_PARAMS);
 
-	drumkv1 *pDrumk = instance();
-	if (pDrumk)
-		pDrumk->reset();
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi)
+		pDrumkUi->reset();
 
 	refreshElements();
 	activateElement();
@@ -794,23 +804,11 @@ void drumkv1widget::loadPreset ( const QString& sFilename )
 	resetParamKnobs(drumkv1::NUM_PARAMS);
 	resetParamValues(drumkv1::NUM_PARAMS);
 
-	drumkv1 *pDrumk = instance();
-	if (pDrumk == NULL)
-		return;
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi)
+		drumkv1_param::loadPreset(pDrumkUi->instance(), sFilename);
 
-	drumkv1_param::loadPreset(pDrumk, sFilename);
-
-	updateParamValues(drumkv1::NUM_PARAMS);
-
-	const QString& sPreset
-		= QFileInfo(sFilename).completeBaseName();
-
-	m_ui.Preset->setPreset(sPreset);
-	m_ui.StatusBar->showMessage(tr("Load preset: %1").arg(sPreset), 5000);
-	updateDirtyPreset(false);
-
-	refreshElements();
-	activateElement();
+	updateLoadPreset(QFileInfo(sFilename).completeBaseName());
 }
 
 
@@ -820,7 +818,9 @@ void drumkv1widget::savePreset ( const QString& sFilename )
 	qDebug("drumkv1widget::savePreset(\"%s\")", sFilename.toUtf8().constData());
 #endif
 
-	drumkv1_param::savePreset(instance(), sFilename);
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi)
+		drumkv1_param::savePreset(pDrumkUi->instance(), sFilename);
 
 	const QString& sPreset
 		= QFileInfo(sFilename).completeBaseName();
@@ -843,7 +843,7 @@ void drumkv1widget::clearSample (void)
 // Sample file loader slot.
 void drumkv1widget::loadSample ( const QString& sFilename )
 {
-	loadSampleFile(sFilename);
+	loadSampleFile(QFileInfo(sFilename).canonicalFilePath());
 
 	m_ui.StatusBar->showMessage(tr("Load sample: %1").arg(sFilename), 5000);
 	updateDirtyPreset(true);
@@ -857,19 +857,6 @@ void drumkv1widget::openSample (void)
 }
 
 
-// Sample updater slot.
-void drumkv1widget::updateSampleNotify (void)
-{
-#ifdef CONFIG_DEBUG
-	qDebug("drumkv1widget::updateSampleNotify()");
-#endif
-
-	drumkv1 *pDrumk = instance();
-	if (pDrumk)
-		updateSample(pDrumk->sample());
-}
-
-
 // Sample file reset.
 void drumkv1widget::clearSampleFile (void)
 {
@@ -877,9 +864,9 @@ void drumkv1widget::clearSampleFile (void)
 	qDebug("drumkv1widget::clearSampleFile()");
 #endif
 
-	drumkv1 *pDrumk = instance();
-	if (pDrumk)
-		pDrumk->setSampleFile(NULL);
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi)
+		pDrumkUi->setSampleFile(NULL);
 
 	updateSample(NULL);
 }
@@ -892,23 +879,23 @@ void drumkv1widget::loadSampleFile ( const QString& sFilename )
 	qDebug("drumkv1widget::loadSampleFile(\"%s\")", sFilename.toUtf8().constData());
 #endif
 
-	drumkv1 *pDrumk = instance();
-	if (pDrumk == NULL)
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi == NULL)
 		return;
 
 	const int note = currentNote();
 	if (note < 0)
 		return;
 
-	drumkv1_element *element = pDrumk->element(note);
+	drumkv1_element *element = pDrumkUi->element(note);
 	if (element == NULL) {
-		element = pDrumk->addElement(note);
+		element = pDrumkUi->addElement(note);
 		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
 			drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-			float fValue = drumkv1_param::paramDefaultValue(index);
+			const float fValue = drumkv1_param::paramDefaultValue(index);
 			element->setParamValue(index, fValue);
 		}
-		pDrumk->setCurrentElement(note);
+		pDrumkUi->setCurrentElement(note);
 		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
 			drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 			setParamValue(index, element->paramValue(index));
@@ -916,8 +903,8 @@ void drumkv1widget::loadSampleFile ( const QString& sFilename )
 		activateParamKnobs(true);
 	}
 
-	pDrumk->setSampleFile(sFilename.toUtf8().constData());
-	updateSample(pDrumk->sample(), true);
+	pDrumkUi->setSampleFile(sFilename.toUtf8().constData());
+	updateSample(pDrumkUi->sample(), true);
 
 	refreshElements();
 }
@@ -1060,7 +1047,7 @@ void drumkv1widget::refreshElements (void)
 	const bool bBlockSignals = m_ui.Elements->blockSignals(true);
 
 	if (m_ui.Elements->instance() == NULL)
-		m_ui.Elements->setInstance(instance());
+		m_ui.Elements->setInstance(ui_instance());
 
 	int iCurrentNote = currentNote();
 
@@ -1081,9 +1068,9 @@ void drumkv1widget::refreshElements (void)
 // All element clear.
 void drumkv1widget::clearElements (void)
 {
-	drumkv1 *pDrumk = instance();
-	if (pDrumk)
-		pDrumk->clearElements();
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi)
+		pDrumkUi->clearElements();
 }
 
 
@@ -1098,22 +1085,22 @@ void drumkv1widget::activateElement ( bool bOpenSample )
 	qDebug("drumkv1widget::activateElement(%d)", iCurrentNote);
 #endif
 
-	drumkv1 *pDrumk = instance();
-	if (pDrumk == NULL)
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi == NULL)
 		return;
 
-	drumkv1_element *element = pDrumk->element(iCurrentNote);
+	drumkv1_element *element = pDrumkUi->element(iCurrentNote);
 	if (element == NULL && bOpenSample) {
-		element = pDrumk->addElement(iCurrentNote);
+		element = pDrumkUi->addElement(iCurrentNote);
 		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
 			drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
-			float fValue = drumkv1_param::paramDefaultValue(index);
+			const float fValue = drumkv1_param::paramDefaultValue(index);
 			element->setParamValue(index, fValue, 0);
 			element->setParamValue(index, fValue);
 		}
 	}
 
-	pDrumk->setCurrentElement(iCurrentNote);
+	pDrumkUi->setCurrentElement(iCurrentNote);
 
 	resetParamKnobs(drumkv1::NUM_ELEMENT_PARAMS);
 
@@ -1123,9 +1110,10 @@ void drumkv1widget::activateElement ( bool bOpenSample )
 			drumkv1widget_knob *pKnob = paramKnob(index);
 			if (pKnob)
 				pKnob->setDefaultValue(element->paramValue(index, 0));
-			setParamValue(index, element->paramValue(index));
+			const float fValue = element->paramValue(index);
+			setParamValue(index, fValue);
 		}
-		updateSample(pDrumk->sample());
+		updateSample(pDrumkUi->sample());
 		refreshElements();
 	} else {
 		updateSample(NULL);
@@ -1142,21 +1130,29 @@ void drumkv1widget::activateElement ( bool bOpenSample )
 }
 
 
-// Element sample loader.
+// Element sample requester.
 void drumkv1widget::doubleClickElement (void)
 {
 	activateElement(true);
 }
 
 
+// Element sample loader.
+void drumkv1widget::loadSampleElement ( const QString& sFilename )
+{
+	emit loadSampleFile(sFilename);
+}
+
+
 // Element deactivation.
 void drumkv1widget::resetElement (void)
 {
 	clearSampleFile();
 
-	drumkv1 *pDrumk = instance();
-	if (pDrumk) {
-		pDrumk->removeElement(pDrumk->currentElement());
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi) {
+		const int key = pDrumkUi->currentElement();
+		pDrumkUi->removeElement(key);
 		updateDirtyPreset(true);
 	}
 
@@ -1195,15 +1191,12 @@ void drumkv1widget::bpmSyncChanged (void)
 		return;
 
 	++m_iUpdate;
-	drumkv1 *pDrumk = instance();
-	if (pDrumk) {
-		float *pBpmSync = pDrumk->paramPort(drumkv1::DEL1_BPMSYNC);
-		if (pBpmSync) {
-			const bool bBpmSync0 = (*pBpmSync > 0.0f);
-			const bool bBpmSync1 = m_ui.Del1BpmKnob->isSpecialValue();
-			if ((bBpmSync1 && !bBpmSync0) || (!bBpmSync1 && bBpmSync0))
-				*pBpmSync = (bBpmSync1 ? 1.0f : 0.0f);
-		}
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi) {
+		const bool bBpmSync0 = (pDrumkUi->paramValue(drumkv1::DEL1_BPMSYNC) > 0.0f);
+		const bool bBpmSync1 = m_ui.Del1BpmKnob->isSpecialValue();
+		if ((bBpmSync1 && !bBpmSync0) || (!bBpmSync1 && bBpmSync0))
+			pDrumkUi->setParamValue(drumkv1::DEL1_BPMSYNC, (bBpmSync1 ? 1.0f : 0.0f));
 	}
 	--m_iUpdate;
 }
@@ -1219,15 +1212,17 @@ void drumkv1widget::contextMenuRequest ( const QPoint& pos )
 	QMenu menu(this);
 	QAction *pAction;
 
-	drumkv1 *pDrumk = instance();
+	drumkv1_ui *pDrumkUi = ui_instance();
 	drumkv1_element *element = NULL;
-	if (pDrumk)
-		element = pDrumk->element(pDrumk->currentElement());
+	if (pDrumkUi) {
+		const int key = pDrumkUi->currentElement();
+		element = pDrumkUi->element(key);
+	}
 
 	pAction = menu.addAction(
 		QIcon(":/images/fileOpen.png"),
 		tr("Open Sample..."), this, SLOT(openSample()));
-	pAction->setEnabled(pDrumk != NULL);
+	pAction->setEnabled(pDrumkUi != NULL);
 	menu.addSeparator();
 	pAction = menu.addAction(
 		tr("Reset"), this, SLOT(resetElement()));
@@ -1242,14 +1237,59 @@ void drumkv1widget::contextMenuRequest ( const QPoint& pos )
 }
 
 
-// Menu actions.
-void drumkv1widget::helpUseNativeDialogs ( bool bOn )
+// Preset status updater.
+void drumkv1widget::updateLoadPreset ( const QString& sPreset )
 {
-	drumkv1widget_config *pConfig = drumkv1widget_config::getInstance();
-	if (pConfig) {
-		pConfig->bUseNativeDialogs = bOn;
-		pConfig->bDontUseNativeDialogs = !pConfig->bUseNativeDialogs;
+//	refreshElements();
+	activateElement();
+
+	updateParamValues(drumkv1::NUM_PARAMS);
+
+	m_ui.Preset->setPreset(sPreset);
+	m_ui.StatusBar->showMessage(tr("Load preset: %1").arg(sPreset), 5000);
+	updateDirtyPreset(false);
+}
+
+
+// Notification updater.
+void drumkv1widget::updateSchedNotify ( int stype )
+{
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi == NULL)
+		return;
+
+#ifdef CONFIG_DEBUG
+	qDebug("drumkv1widget::updateSchedNotify(%d)", stype);
+#endif
+
+	switch (drumkv1_sched::Type(stype)) {
+	case drumkv1_sched::Programs: {
+		drumkv1_programs *pPrograms = pDrumkUi->programs();
+		drumkv1_programs::Prog *pProg = pPrograms->current_prog();
+		if (pProg) updateLoadPreset(pProg->name());
+		break;
 	}
+	case drumkv1_sched::Sample:
+		updateSample(pDrumkUi->sample());
+		// Fall thru...
+	default:
+		break;
+	}
+}
+
+
+// Menu actions.
+void drumkv1widget::helpConfigure (void)
+{
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi == NULL)
+		return;
+
+	drumkv1widget_config form(this);
+
+	// Set programs database...
+	form.setPrograms(pDrumkUi->programs());
+	form.exec();
 }
 
 
