@@ -1,7 +1,7 @@
 // drumkv1widget.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -24,9 +24,9 @@
 
 #include "ui_drumkv1widget.h"
 
-#include "drumkv1widget_config.h"
+#include "drumkv1_config.h"
 
-#include "drumkv1.h"
+#include "drumkv1_ui.h"
 
 
 // forward decls.
@@ -103,15 +103,15 @@ protected slots:
 	// Element activation.
 	void activateElement(bool bOpenSample = false);
 
-	// Element sample loader.
+	// Element sample requester.
 	void doubleClickElement();
 
+	// Element sample loader.
+	void loadSampleElement(const QString& sFilename);
+
 	// Element deactivation.
 	void resetElement(void);
 
-	// Sample updater.
-	void updateSampleNotify();
-
 	// Common context menu.
 	void contextMenuRequest(const QPoint& pos);
 
@@ -124,8 +124,11 @@ protected slots:
 	// Delay BPM change.
 	void bpmSyncChanged();
 
+	// Notification updater.
+	void updateSchedNotify(int stype);
+
 	// Menu actions.
-	void helpUseNativeDialogs(bool bOn);
+	void helpConfigure();
 
 	void helpAbout();
 	void helpAboutQt();
@@ -133,7 +136,7 @@ protected slots:
 protected:
 
 	// Synth engine accessor.
-	virtual drumkv1 *instance() const = 0;
+	virtual drumkv1_ui *ui_instance() const = 0;
 
 	// Reload all elements.
 	void refreshElements();
@@ -167,6 +170,9 @@ protected:
 	// Update local tied widgets.
 	void updateParamEx(drumkv1::ParamIndex index, float fValue);
 
+	// Preset status updater.
+	void updateLoadPreset(const QString& sPreset);
+
 	// Dirty flag (overridable virtual) methods.
 	virtual void updateDirtyPreset(bool bDirtyPreset);
 
@@ -175,8 +181,6 @@ private:
 	// Instance variables.
 	Ui::drumkv1widget m_ui;
 
-	drumkv1widget_config m_config;
-
 	drumkv1_sched_notifier *m_sched_notifier;
 
 	QHash<drumkv1::ParamIndex, drumkv1widget_knob *> m_paramKnobs;
