@@ -1,7 +1,7 @@
 // drumkv1widget_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -32,6 +32,22 @@
 
 #include <QCloseEvent>
 
+#include <QStyleFactory>
+
+#ifndef CONFIG_LIBDIR
+#if defined(__x86_64__)
+#define CONFIG_LIBDIR CONFIG_PREFIX "/lib64"
+#else
+#define CONFIG_LIBDIR CONFIG_PREFIX "/lib"
+#endif
+#endif
+
+#if QT_VERSION < 0x050000
+#define CONFIG_PLUGINSDIR CONFIG_LIBDIR "/qt4/plugins"
+#else
+#define CONFIG_PLUGINSDIR CONFIG_LIBDIR "/qt5/plugins"
+#endif
+
 
 //-------------------------------------------------------------------------
 // drumkv1widget_jack - impl.
@@ -44,19 +60,38 @@ drumkv1widget_jack::drumkv1widget_jack ( drumkv1_jack *pDrumk )
 		, m_pNsmClient(NULL)
 	#endif
 {
+	// Special style paths...
+	if (QDir(CONFIG_PLUGINSDIR).exists())
+		QApplication::addLibraryPath(CONFIG_PLUGINSDIR);
+
+	// Custom style theme...
+	drumkv1_config *pConfig = drumkv1_config::getInstance();
+	if (pConfig && !pConfig->sCustomStyleTheme.isEmpty())
+		QApplication::setStyle(QStyleFactory::create(pConfig->sCustomStyleTheme));
+
+	// Initialize (user) interface stuff...
+	m_pDrumkUi = new drumkv1_ui(m_pDrumk);
+
 	// Initialize preset stuff...
 	//initPreset();
-	updateParamValues(drumkv1::NUM_PARAMS);
-
 	refreshElements();
 	activateElement();
+
+	updateParamValues(drumkv1::NUM_PARAMS);
+}
+
+
+// Destructor.
+drumkv1widget_jack::~drumkv1widget_jack (void)
+{
+	delete m_pDrumkUi;
 }
 
 
 // Synth engine accessor.
-drumkv1 *drumkv1widget_jack::instance (void) const
+drumkv1_ui *drumkv1widget_jack::ui_instance (void) const
 {
-	return m_pDrumk;
+	return m_pDrumkUi;
 }
 
 #ifdef CONFIG_NSM
@@ -66,7 +101,7 @@ void drumkv1widget_jack::setNsmClient ( drumkv1_nsm *pNsmClient )
 {
 	m_pNsmClient = pNsmClient;
 
-	drumkv1widget_config *pConfig = drumkv1widget_config::getInstance();
+	drumkv1_config *pConfig = drumkv1_config::getInstance();
 	if (pConfig)
 		pConfig->bDontUseNativeDialogs = true;
 }
@@ -83,9 +118,7 @@ drumkv1_nsm *drumkv1widget_jack::nsmClient (void) const
 void drumkv1widget_jack::updateParam (
 	drumkv1::ParamIndex index, float fValue ) const
 {
-	float *pParamPort = m_pDrumk->paramPort(index);
-	if (pParamPort)
-		*pParamPort = fValue;
+	m_pDrumkUi->setParamValue(index, fValue);
 }
 
 
