@@ -1,7 +1,7 @@
 // drumkv1widget_knob.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -22,7 +22,6 @@
 #include "drumkv1widget_knob.h"
 
 #include <QLabel>
-#include <QDial>
 #include <QSpinBox>
 #include <QComboBox>
 
@@ -37,6 +36,105 @@
 inline int iroundf(float x) { return int(x < 0.0f ? x - 0.5f : x + 0.5f); }
 
 
+//-------------------------------------------------------------------------
+// drumkv1widget_dial - A better QDial widget.
+
+drumkv1widget_dial::DialMode
+drumkv1widget_dial::g_dialMode = drumkv1widget_dial::DefaultMode;
+
+// Set knob dial mode behavior.
+void drumkv1widget_dial::setDialMode ( DialMode dialMode )
+	{ g_dialMode = dialMode; }
+
+drumkv1widget_dial::DialMode drumkv1widget_dial::dialMode (void)
+	{ return g_dialMode; }
+
+
+// Constructor.
+drumkv1widget_dial::drumkv1widget_dial ( QWidget *pParent )
+	: QDial(pParent), m_bMousePressed(false), m_fLastDragValue(0.0f)
+{
+}
+
+
+// Mouse angle determination.
+float drumkv1widget_dial::mouseAngle ( const QPoint& pos )
+{
+	const float dx = pos.x() - (width() >> 1);
+	const float dy = (height() >> 1) - pos.y();
+	return 180.0f * ::atan2f(dx, dy) / float(M_PI);
+}
+
+
+// Alternate mouse behavior event handlers.
+void drumkv1widget_dial::mousePressEvent ( QMouseEvent *pMouseEvent )
+{
+	if (g_dialMode == DefaultMode) {
+		QDial::mousePressEvent(pMouseEvent);
+	} else if (pMouseEvent->button() == Qt::LeftButton) {
+		m_bMousePressed = true;
+		m_posMouse = pMouseEvent->pos();
+		m_fLastDragValue = float(value());
+		emit sliderPressed();
+	}
+}
+
+
+void drumkv1widget_dial::mouseMoveEvent ( QMouseEvent *pMouseEvent )
+{
+	if (g_dialMode == DefaultMode) {
+		QDial::mouseMoveEvent(pMouseEvent);
+		return;
+	}
+
+	if (!m_bMousePressed)
+		return;
+
+	const QPoint& pos = pMouseEvent->pos();
+	const int dx = pos.x() - m_posMouse.x();
+	const int dy = pos.y() - m_posMouse.y();
+	float fAngleDelta =  mouseAngle(pos) - mouseAngle(m_posMouse);
+	int iNewValue = value();
+
+	switch (g_dialMode)	{
+	case LinearMode:
+		iNewValue = int(m_fLastDragValue) + dx - dy;
+		break;
+	case AngularMode:
+	default:
+		// Forget about the drag origin to be robust on full rotations
+		if (fAngleDelta > +180.0f) fAngleDelta -= 360.0f;
+		else
+		if (fAngleDelta < -180.0f) fAngleDelta += 360.0f;
+		m_fLastDragValue += float(maximum() - minimum()) * fAngleDelta / 270.0f;
+		if (m_fLastDragValue > float(maximum()))
+			m_fLastDragValue = float(maximum());
+		else
+		if (m_fLastDragValue < float(minimum()))
+			m_fLastDragValue = float(minimum());
+		m_posMouse = pos;
+		iNewValue = int(m_fLastDragValue + 0.5f);
+		break;
+	}
+
+	setValue(iNewValue);
+	update();
+
+	emit sliderMoved(value());
+}
+
+
+void drumkv1widget_dial::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
+{
+	if (g_dialMode == DefaultMode
+		&& pMouseEvent->button() != Qt::MidButton) {
+		QDial::mouseReleaseEvent(pMouseEvent);
+	} else if (m_bMousePressed) {
+		m_bMousePressed = false;
+	}
+}
+
+
 //-------------------------------------------------------------------------
 // drumkv1widget_knob - Custom composite widget.
 //
@@ -49,7 +147,7 @@ drumkv1widget_knob::drumkv1widget_knob ( QWidget *pParent ) : QWidget(pParent)
 	QWidget::setFont(font2);
 
 	m_pLabel = new QLabel();
-	m_pDial  = new QDial();
+	m_pDial  = new drumkv1widget_dial();
 
 	m_fScale = 100.0f;
 
@@ -99,7 +197,7 @@ void drumkv1widget_knob::setValue ( float fValue, bool bDefault )
 	}
 	else
 	if (QWidget::isEnabled()
-		&& ::fabs(fValue - m_fDefaultValue) > 0.001f) {
+		&& ::fabs(fValue - m_fDefaultValue) > 0.0001f) {
 		pal.setColor(QPalette::Base,
 			(pal.window().color().value() < 0x7f
 				? QColor(Qt::darkYellow).darker()
@@ -297,13 +395,13 @@ void drumkv1widget_spin::setSingleStep ( float fSingleStep )
 
 QString drumkv1widget_spin::valueText (void) const
 {
-	return QString::number(m_pSpinBox->value());
+	return QString::number(value());
 }
 
 
 float drumkv1widget_spin::value (void) const
 {
-	return float(m_pSpinBox->value());
+	return valueFromScale(m_pSpinBox->value());
 }
 
 
